package com.sq.customization.util;

public class StringConditionChecker {

    public static void main(String[] args) {
        // 示例测试
        String condition = "%abc & !=def | (%xyz & ==abc)"; // 条件示例
        String str = "This is abc text"; // 目标字符串
        boolean result = checkCondition(condition, str);
        System.out.println(result ? "条件符合" : "条件不符合");
    }

    /**
     * 检查条件是否符合
     * @param condition 条件字符串
     * @param str 目标字符串
     * @return 是否符合条件
     */
    public static boolean checkCondition(String condition, String str) {
        condition = condition.trim(); // 去掉首尾空格

        // 处理括号
        if (condition.startsWith("(") && condition.endsWith(")")) {
            return checkCondition(condition.substring(1, condition.length() - 1), str);
        }

        // 处理多个 & 和 | 运算符
        int[] operatorIndices = findAllOuterOperators(condition);
        if (operatorIndices.length > 0) {
            String[] parts = splitCondition(condition, operatorIndices);
            char[] operators = getOperators(condition, operatorIndices);

            boolean result = checkCondition(parts[0], str);
            int partIndex = 1;
            for (int i = 0; i < operators.length; i++) {
                boolean nextResult = checkCondition(parts[partIndex++], str);
                if (operators[i] == '&') {
                    result = result && nextResult;
                } else if (operators[i] == '|') {
                    result = result || nextResult;
                }
            }
            return result;
        }

        // 处理单个条件
        return checkSingleCondition(condition, str);
    }

    /**
     * 查找所有最外层的 & 和 | 运算符
     */
    private static int[] findAllOuterOperators(String condition) {
        int parenthesesLevel = 0;
        java.util.List<Integer> indices = new java.util.ArrayList<>();
        for (int i = 0; i < condition.length(); i++) {
            char c = condition.charAt(i);
            if (c == '(') {
                parenthesesLevel++;
            } else if (c == ')') {
                parenthesesLevel--;
            } else if (parenthesesLevel == 0 && (c == '&' || c == '|')) {
                indices.add(i);
            }
        }
        int[] result = new int[indices.size()];
        for (int i = 0; i < indices.size(); i++) {
            result[i] = indices.get(i);
        }
        return result;
    }

    /**
     * 根据运算符位置分割条件
     */
    private static String[] splitCondition(String condition, int[] operatorIndices) {
        String[] parts = new String[operatorIndices.length + 1];
        int start = 0;
        for (int i = 0; i < operatorIndices.length; i++) {
            parts[i] = condition.substring(start, operatorIndices[i]).trim();
            start = operatorIndices[i] + 1;
        }
        parts[operatorIndices.length] = condition.substring(start).trim();
        return parts;
    }

    /**
     * 获取所有运算符
     */
    private static char[] getOperators(String condition, int[] operatorIndices) {
        char[] operators = new char[operatorIndices.length];
        for (int i = 0; i < operatorIndices.length; i++) {
            operators[i] = condition.charAt(operatorIndices[i]);
        }
        return operators;
    }

    /**
     * 检查单个条件
     */
    private static boolean checkSingleCondition(String condition, String str) {
        condition = condition.trim();

        // 从长到短检查运算符，避免冲突
        if (condition.startsWith("!%")) {
            String substring = condition.substring(2);
            return !str.contains(substring); // 不包含
        } else if (condition.startsWith("%")) {
            String substring = condition.substring(1);
            return str.contains(substring); // 包含
        } else if (condition.startsWith("==")) {
            String operand = condition.substring(2);
            return str.equals(operand); // 等于
        } else if (condition.startsWith("!=")) {
            String operand = condition.substring(2);
            return !str.equals(operand); // 不等于
        }

        return false; // 未识别的条件返回 false
    }
}

